/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.fmubuilder;

import org.javafmi.kernel.operatingsystem.Zipper;

import java.io.*;

import static org.javafmi.kernel.operatingsystem.FileUtils.*;

public class FmuBuilder {

    private final File workingDirectory;

    public FmuBuilder() {
        this.workingDirectory = new File("fmu_build_folder");
        this.workingDirectory.mkdirs();
        createFolder(workingDirectory, "resources");
    }

    public void addModelDescription(File modelDescription) {
        BufferedInputStream source = bufferedInputStream(fileInputStream(modelDescription));
        BufferedOutputStream target = bufferedOutputStream(fileOutputStream(createFile(workingDirectory, "modelDescription.xml")));
        copyFromTo(source, target);
        close(source);
        close(target);
    }

    public void addLibrary(Library library, String directory) {
        BufferedInputStream source = bufferedInputStream(library.getInputStream());
        BufferedOutputStream target = bufferedOutputStream(fileOutputStream(createFile(createFolder(workingDirectory, directory), library.getName())));
        copyFromTo(source, target);
        close(source);
        close(target);
    }

    public void addFileAsResource(File resourceName) {
        FileInputStream source = fileInputStream(resourceName);
        FileOutputStream target = fileOutputStream(createFile(createFolder(workingDirectory, "resources"), resourceName.getName()));
        copyFromTo(source, target);
        close(source);
        close(target);
    }

    public File buildFmu(String fmuName) {
        if(!fmuName.endsWith(".fmu")) fmuName = fmuName + ".fmu";
        File fmu = new Zipper().zipDirectory(workingDirectory, fmuName);
        deleteAll(workingDirectory);
        return fmu;
    }

    public File getWorkingDirectory() {
        return this.workingDirectory;
    }
}
