/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */

import org.javafmi.fmubuilder.FmuBuilder;
import org.javafmi.kernel.operatingsystem.UnZipper;
import org.junit.Test;

import java.io.File;
import java.io.IOException;

import static org.junit.Assert.assertEquals;

public class FMUBuilderTest {
    private static final String MODEL_DESCRIPTION = "BuilderTest.xml";
    public static String FMU_NAME = "FmuBuilderTest.fmu";
    public static String[] RESOURCES = new String[]{"rt.jar", "FmuBuilderTest.txt"};

    @Test
    public void buildingAnEmptyFMU() {
        FmuBuilder fmuBuilder = new FmuBuilder();
        File emptyFmu = fmuBuilder.buildFmu(FMU_NAME);
        assertEquals(FMU_NAME, emptyFmu.getName());
        assertEquals(true, (emptyFmu.listFiles() == null));

    }

    @Test
    public void buildingFmuWithOneResource() {
        FmuBuilder fmuBuilder = new FmuBuilder();
        fmuBuilder.addFileAsResource(new File(RESOURCES[0]));
        File fmuFile = fmuBuilder.buildFmu(FMU_NAME);
        UnZipper unZipper = new UnZipper(fmuFile);
        assertEquals((new File(RESOURCES[0])).length(), unZipper.unzip("resources/" + RESOURCES[0], new File("temp.jar")).length());
    }


    @Test
    public void buildingFmuWithXml() throws IOException {
        FmuBuilder fmuBuilder = new FmuBuilder();
        fmuBuilder.addModelDescription(new File(MODEL_DESCRIPTION));
        File fmuFile = fmuBuilder.buildFmu(FMU_NAME);
        UnZipper unZipper = new UnZipper(fmuFile);
        //  assertEquals((new File(MODEL_DESCRIPTION)).length(), unZipper.unzip(MODEL_DESCRIPTION,new File(Files.createTempFile("builder","test"))).length());

    }
}