/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.kernel;

import org.javafmi.kernel.operatingsystem.FmuFile;
import org.javafmi.kernel.operatingsystem.PathBuilder;

import java.io.File;

import static org.javafmi.kernel.operatingsystem.SystemUtils.systemIsLinux32;
import static org.javafmi.kernel.operatingsystem.SystemUtils.systemIsWin32;

public class CommonsKernel {

    public static final String WINDOWS_32_V1_FMU = "win_32_SFR_CoSimulation_V1.fmu";
    public static final String WINDOWS_32_V2RC1_FMU = "win_32_64_HTB_0REUNION_V2RC1.fmu";
    public static final String WINDOWS_32_64_FMU_WITH_DIRECTIONAL_DERIVATIVES = "PackageBuildingCases_ComponentsFMU_wallFmu1.fmu";
    public static final String WINDOWS_32_INC_FMU = "win_32_inc.fmu";
    public static final String WINDOWS_32_BOUNCING_BALL_FMU = "win_32_bouncingBall.fmu";
    public static final String WINDOWS_32_DEBUGGABLE_FMU = "win_32_bouncingBall.fmu";
    public static final String WINDOWS_32_REAL_READABLE_FMU = "win_32_bouncingBall.fmu";
    public static final String WINDOWS_32_INTEGER_READABLE_FMU = "win_32_inc.fmu";
    public static final String WINDOWS_32_STRING_READABLE_FMU = "win_32_values.fmu";
    public static final String WINDOWS_32_BOOLEAN_READABLE_FMU = "win_32_values.fmu";
    public static final String WINDOWS_32_ENUMERATION_READABLE_FMU = "win_32_GridSysPro_0V1_Examples_Validation_0with_0generator.fmu";
    public static final String WINDOWS_32_REAL_WRITABLE_FMU = "win_32_bouncingBall.fmu";
    public static final String WINDOWS_32_INTEGER_WRITABLE_FMU = "win_32_values.fmu";
    public static final String LINUX_32_FMU = "linux_32_plusun.fmu";
    public static final String LINUX_32_PLUSUN_FMU = "linux_32_plusun.fmu";
    public static final String LINUX_64_V2RC2_FMU = "linux_64_V2RC2_input_output_tankv5.fmu";
    public static final String LINUX_64_V2_FMU = "linux_64_V2_tankv3.fmu";
    public static final String FMU_WITH_RESOURCES = "fmu_with_resources.fmu";
    public static final String FMU_WITH_RESOURCES_BUT_EMPTY = "fmu_with_resources_but_empty.fmu";
    public static final String ERROR_MESSAGE_FOR_NOT_DEFINED_ARCHITECTURE = "No test defined for your machine: " + System.getProperty("os.name") + " " + System.getProperty("os.arch");

    public static FmuFile createFmuFile(String fmuName) {
        return new FmuFile(new File(getFmuPath(fmuName)));
    }

    public static String getFmuPath(String fmuName) {
        return PathBuilder.buildPath(new String[]{"../test resources", "fmu", "cosimulation"}, fmuName);
    }

    public static FmuProxy createProxy(String fmuName) {
        return new org.javafmi.kernel.v1.FmuProxy(createFmuFile(fmuName));
    }

    public static FmuProxy createV2FmuProxy(String fmuName) {
        return new org.javafmi.kernel.v2.FmuProxy(createFmuFile(fmuName));
    }

    public static FmuProxy createV2RC2FmuProxy(String fmuName) {
        return new org.javafmi.kernel.v2.FmuProxy(createFmuFile(fmuName));
    }

    public static String getFmuNameForThisOS() {
        if(systemIsWin32()) return WINDOWS_32_V1_FMU;
        if(systemIsLinux32()) return LINUX_32_FMU;
        throw new RuntimeException("There is not an FMU for this system");
    }

    public static String getV2FmuNameForThisOS() {
        if(systemIsWin32()) return WINDOWS_32_V2RC1_FMU;
        return null;
    }
}
