/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.kernel;

import org.javafmi.kernel.operatingsystem.FmuFile;
import org.javafmi.modeldescription.FmiVersion;
import org.junit.After;
import org.junit.Test;

import static org.javafmi.kernel.CommonsKernel.*;
import static org.javafmi.kernel.operatingsystem.SystemUtils.*;
import static org.junit.Assert.assertEquals;
import static org.junit.Assume.assumeTrue;

public class FmuProxyFactoryTest {

    private FmuFile fmuFile;

    @After
    public void cleanUp() {
        if(fmuFile!= null) deleteAll(fmuFile.getWorkingDirectoryPath());
    }

    @Test
    public void testFactoryForV1Fmu() {
        assumeTrue(systemIsWin32());
        fmuFile = createFmuFile(WINDOWS_32_V1_FMU);
        FmuProxy proxy = ProxyFactory.createProxyFor(fmuFile);
        assertEquals(org.javafmi.kernel.v1.FmuProxy.class, proxy.getClass());
        assertEquals(FmiVersion.ONE, proxy.getVersion());
    }

    @Test
    public void testFactoryForV2Fmu() {
        assumeTrue(systemIsLinux64());
        fmuFile = createFmuFile(LINUX_64_V2RC2_FMU);
        FmuProxy proxy = ProxyFactory.createProxyFor(fmuFile);
        assertEquals(org.javafmi.kernel.v2.FmuProxy.class, proxy.getClass());
        assertEquals(FmiVersion.TWO, proxy.getVersion());
    }
}
