/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.kernel.v1;

import org.javafmi.kernel.FmuProxy;
import org.javafmi.kernel.Status;
import org.junit.Test;

import static org.javafmi.kernel.CommonsKernel.*;
import static org.javafmi.kernel.operatingsystem.SystemUtils.systemIsLinux32;
import static org.javafmi.kernel.operatingsystem.SystemUtils.systemIsWin32;
import static org.junit.Assert.*;
import static org.junit.Assume.assumeTrue;

public class FmuProxyCreationAndDestructionTest {

    private static final double TIMEOUT = 1000.;
    private static final double START_TIME = 0.0;
    private static final double STOP_TIME = 10.0;
    private static final double STEP_SIZE = 0.2;

    @Test
    public void createAndDestroyInstance() {
        assumeTrue(systemIsWin32() || systemIsLinux32());
        FmuProxy fmuProxy = createProxy(getFmuNameForThisOS());
        fmuProxy.instantiate(TIMEOUT);
        fmuProxy.initialize(START_TIME, STOP_TIME);
        Status terminateStatus = fmuProxy.terminate();
        fmuProxy.freeInstance();
        assertEquals(Status.OK, terminateStatus);
    }

    @Test
    public void testSetDebugSlave() {
        assumeTrue(systemIsWin32());
        testDebugForCapableWn32Fmu();
    }

    @Test
    public void testDoStepAndReset() {
        assumeTrue(systemIsWin32() || systemIsLinux32());
        FmuProxy fmuProxy = createProxy(getFmuNameForThisOS());
        fmuProxy.instantiate(TIMEOUT);
        fmuProxy.initialize(START_TIME, STOP_TIME);
        fmuProxy.doStep(START_TIME, STEP_SIZE);
        Status resetStatus = fmuProxy.reset();
        fmuProxy.terminate();
        fmuProxy.freeInstance();

        assertEquals(Status.OK, resetStatus);
    }

    private void testDebugForCapableWn32Fmu() {
        assumeTrue(systemIsWin32());
        org.javafmi.kernel.v1.FmuProxy proxy = (org.javafmi.kernel.v1.FmuProxy) createProxy(WINDOWS_32_DEBUGGABLE_FMU);
        proxy.instantiate(TIMEOUT);
        boolean doingLoggingBeforeSet = proxy.isDoingLogging();
        Status setLoggingStatus = proxy.setIsDoingLogging(true);
        boolean doingLoggingAfterSet = proxy.isDoingLogging();
        proxy.terminate();
        proxy.freeInstance();

        assertFalse(doingLoggingBeforeSet);
        assertEquals(Status.OK, setLoggingStatus);
        assertTrue(doingLoggingAfterSet);
    }
}
