/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.kernel.v2;

import org.javafmi.kernel.Status;
import org.javafmi.kernel.operatingsystem.FmuFile;
import org.javafmi.kernel.operatingsystem.SystemUtils;
import org.junit.Test;

import static org.javafmi.kernel.CommonsKernel.LINUX_64_V2_FMU;
import static org.junit.Assert.assertEquals;
import static org.junit.Assume.assumeTrue;

public class FmuProxyCreationAndDestructionTest {

    public static final boolean STOP_TIME_DEFINED = true;
    public static final boolean LOGGING_ON = true;
    private static final double TIMEOUT = 1000;
    private static final double START_TIME = 0.;
    private static final double STOP_TIME = 10.;
    private static final boolean TOLERANCE_DEFINED = false;

    @Test
    public void createAndDestroyLinux64Instance() {
        assumeTrue(SystemUtils.systemIsLinux64());
        FmuProxy proxy = new FmuProxy(new FmuFile(LINUX_64_V2_FMU));
        double tolerance = 0.001;
        proxy.instantiate(TIMEOUT);
        Status setUpExperimentStatus = proxy.setUpExperiment(TOLERANCE_DEFINED, tolerance, START_TIME, STOP_TIME_DEFINED, STOP_TIME);
        Status enterInitStatus = proxy.enterInitializationMode();
        Status exitInitStatus = proxy.exitInitializationMode();
        Status terminateStatus = proxy.terminate();
        proxy.freeInstance();

        assertEquals(Status.OK, setUpExperimentStatus);
        assertEquals(Status.OK, enterInitStatus);
        assertEquals(Status.OK, exitInitStatus);
        assertEquals(Status.OK, terminateStatus);
    }

    @Test
    public void testSetDebugLoggingInLinux64() {
        assumeTrue(SystemUtils.systemIsLinux64());
        FmuProxy proxy = new FmuProxy(new FmuFile(LINUX_64_V2_FMU));
        String categories[] = {"Severe", "Warning"};
        proxy.instantiate(TIMEOUT);
        Status enterInitStatus = proxy.enterInitializationMode();
        Status exitInitStatus = proxy.exitInitializationMode();
        Status setDebugStatus = proxy.setDebugLogging(LOGGING_ON, categories);
        Status terminateStatus = proxy.terminate();
        proxy.freeInstance();

        assertEquals(Status.OK, enterInitStatus);
        assertEquals(Status.OK, exitInitStatus);
        assertEquals(Status.OK, setDebugStatus);
        assertEquals(Status.OK, terminateStatus);
    }
}
