/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.kernel.v2;


import org.javafmi.kernel.CommonsKernel;
import org.javafmi.kernel.operatingsystem.SystemUtils;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assume.assumeTrue;

public class FmuProxyGettingAndSettingVariablesTest {
    private static final double TIMEOUT = 1000.;
    private FmuProxy proxy;

    @Before
    public void setUp() {
        assumeTrue(SystemUtils.systemIsLinux64());
        proxy = (FmuProxy) CommonsKernel.createV2RC2FmuProxy(CommonsKernel.LINUX_64_V2_FMU);
    }

    @Test
    public void testGetReal() {
        proxy.instantiate(TIMEOUT);
        proxy.enterInitializationMode();
        Double doubleVariable = proxy.getReal(new int[]{3})[0];
        proxy.exitInitializationMode();
        proxy.terminate();
        proxy.freeInstance();
        assertEquals(0.5, doubleVariable, 0.001);
    }

    @Test
    public void callToSetRealSeveralTimesDoesNotCauseOverFlow() {
        proxy.instantiate(TIMEOUT);
        proxy.enterInitializationMode();
        proxy.exitInitializationMode();
        proxy.setReal(new int[]{28}, new double[]{10.0});
        proxy.setReal(new int[]{28}, new double[]{9.0});
        double realValue = proxy.getReal(28)[0];
        proxy.terminate();
        proxy.freeInstance();
        assertEquals(9.0, realValue, 0.001);
    }

    @Test
    public void testGetInteger() {
        proxy.instantiate(TIMEOUT);
        proxy.enterInitializationMode();
        proxy.exitInitializationMode();
        int intVariable = proxy.getInteger(268435470);
        proxy.terminate();
        proxy.freeInstance();
        assertEquals(0, intVariable);
    }

    @Test
    public void testGetBoolean() {
        int booleanVariableValueReference = 536870932;
        proxy.instantiate(TIMEOUT);
        proxy.enterInitializationMode();
        proxy.exitInitializationMode();
        boolean booleanVariable = proxy.getBoolean(booleanVariableValueReference);
        proxy.terminate();
        proxy.freeInstance();
        assertEquals(false, booleanVariable);
    }
}
