/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.kernel.v2;

import org.javafmi.kernel.operatingsystem.FmuFile;
import org.junit.Test;

import static org.javafmi.kernel.CommonsKernel.WINDOWS_32_64_FMU_WITH_DIRECTIONAL_DERIVATIVES;
import static org.javafmi.kernel.operatingsystem.SystemUtils.systemIsWin32;
import static org.javafmi.kernel.operatingsystem.SystemUtils.systemIsWin64;
import static org.junit.Assert.assertEquals;
import static org.junit.Assume.assumeTrue;


public class GettingPartialDerivativesTest {

    @Test
    public void testGetDirectionalDerivative() {
        assumeTrue(systemIsWin32() || systemIsWin64());
        int[] unknownVariablesValueReferences = new int[1];
        int[] knownVariablesValueReferences = new int[1];
        double[] knownDifferential = new double[1];
        FmuProxy proxy = new FmuProxy(new FmuFile(WINDOWS_32_64_FMU_WITH_DIRECTIONAL_DERIVATIVES));
        proxy.instantiate(1000.);
        proxy.enterInitializationMode();
        proxy.exitInitializationMode();
        double[] directionalDerivatives = proxy.getDirectionalDerivative(unknownVariablesValueReferences, knownVariablesValueReferences, knownDifferential);
        proxy.terminate();
        proxy.freeInstance();
        assertEquals(knownDifferential.length, directionalDerivatives.length);
    }
}
