/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi;

import org.javafmi.kernel.operatingsystem.PathBuilder;

import static org.javafmi.kernel.operatingsystem.SystemUtils.systemIsLinux32;
import static org.javafmi.kernel.operatingsystem.SystemUtils.systemIsWin32;
import static org.junit.Assume.assumeTrue;

public class CommonsWrapper {
    public static final String WINDOWS_32_V1_FMU = "win_32_SFR_CoSimulation_V1.fmu";
    public static final String LINUX_32_FMU = "linux_32_plusun.fmu";
    public static final String LINUX_64_V2RC2_FMU = "linux_64_V2RC2_input_output_tankv5.fmu";
    public static final String LINUX_64_V2_FMU = "linux_64_V2_tankv3.fmu";
    public static final String WINDOWS_32_64_FMU_WITH_SEVERAL_VARIABLES = "windows_32_64_fmu_with_several_variables.fmu";
    public static final String WINDOWS_32_64_FMU_WITH_SEVERAL_IO = "windows_32_64_fmu_with_several_IO.fmu";
    public static final String WINDOWS_32_64_FMU_WITH_DIRECTIONAL_DERIVATIVES = "PackageBuildingCases_ComponentsFMU_wallFmu1.fmu";
    public static final String WINDOWS_32_ENUMERATION_READABLE_FMU = "win_32_GridSysPro_0V1_Examples_Validation_0with_0generator.fmu";
    public static final String PACKAGE_BUILDING_CASES = "win_32_64_PackageBuildingCases.fmu";
    public static final String WINDOWS_32_V2RC1_FMU = "win_32_64_HTB_0REUNION_V2RC1.fmu";

    public static Simulation createSimulation(String fmuName) {
        return new Simulation(getFmuPath(fmuName));
    }

    public static String getFmuPath(String fmuName) {
        return PathBuilder.buildPath(new String[]{"../test resources", "fmu", "cosimulation"}, fmuName);
    }

    public static String getFmuNameForThisOS() {
        assumeTrue(systemIsWin32() || systemIsLinux32());
        if (systemIsWin32()) return WINDOWS_32_V1_FMU;
        if (systemIsLinux32()) return LINUX_32_FMU;
        throw new RuntimeException("There is not an FMU for this system");
    }
}
