/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.v2;

import org.javafmi.Simulation;
import org.javafmi.kernel.Status;
import org.junit.Test;

import static org.javafmi.CommonsWrapper.*;
import static org.junit.Assert.assertEquals;

public class SimulationTest {

    private static final double START_TIME = 0.0;
    private static final double TIME_STEP = 0.01;
    private static final double STOP_TIME = 10.0;

    public void doubleInitShouldNotHaveEffect() {
        Simulation simulation = createSimulation("");
        Status firstInitStatus = simulation.init(START_TIME);
        Status secondInitStatus = simulation.init(START_TIME);
        simulation.terminate();

        assertEquals(Status.OK, firstInitStatus);
        assertEquals(Status.WARNING, secondInitStatus);
    }

    public void testReset() {
        Simulation simulation = createSimulation("");
        Status firstInitStatus = simulation.init(START_TIME);
        double firstSimulationTime = simulation.getSimulationTime();
        Status resetStatus = simulation.reset();
        Status secondInitStatus = simulation.init(START_TIME, STOP_TIME);
        Status terminateStatus = simulation.terminate();
        assertEquals(Status.OK, firstInitStatus);
        assertEquals(TIME_STEP, firstSimulationTime, 0.1);
        assertEquals(Status.OK, resetStatus);
        assertEquals(Status.OK, secondInitStatus);
        assertEquals(Status.OK, terminateStatus);
    }


    public void definingAToleranceForASimulation() {
        Simulation simulation = createSimulation(WINDOWS_32_V2RC1_FMU);
        simulation.setSimulationTolerance(1e-3);
        Status initStatus = simulation.init(START_TIME);
        simulation.terminate();
        assertEquals(Status.OK, initStatus);
    }

}
