/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.v2;

import org.javafmi.VariableReader;
import org.javafmi.modeldescription.ModelDescription;
import org.javafmi.modeldescription.v2.*;
import org.junit.Before;
import org.junit.Test;

import static org.mockito.Mockito.*;

public class VariableReaderTest {

    private VariableReader reader;
    private org.javafmi.kernel.FmuProxy mockProxy;
    private ModelDescription mockedModelDescription;

    @Before
    public void setUp() {
        mockProxy = mock(org.javafmi.kernel.v2.FmuProxy.class);
        mockedModelDescription = mock(ModelDescription.class);
        reader = new VariableReader(mockProxy, mockedModelDescription);
    }

    @Test
    public void readingARealCallsToGetRealInProxy() {
        String realVariableName = "";
        int[] realVariableValueReference = new int[]{0};
        configureTest(new RealVariable(realVariableName).withValueReference(realVariableValueReference[0]));
        verify(mockProxy).getReal(realVariableValueReference);
    }

    @Test
    public void readingAnIntegerCallsToGetIntegerInProxy() {
        String intVariableName = "";
        Integer intVariableValueReference = 0;
        configureTest(new IntegerVariable(intVariableName).withValueReference(intVariableValueReference));
        verify(mockProxy).getInteger(intVariableValueReference);
    }

    @Test
    public void readingABooleanCallsToGetBooleanInProxy() {
        String booleanVariableName = "";
        Integer booleanVariableValueReference = 0;
        configureTest(new BooleanVariable(booleanVariableName).withValueReference(booleanVariableValueReference));
        verify(mockProxy).getBoolean(booleanVariableValueReference);
    }

    @Test
    public void readingAStringCallsToGetStringInProxy() {
        String stringVariableName = "";
        Integer stringVariableValueReference = 0;
        configureTest(new StringVariable(stringVariableName).withValueReference(stringVariableValueReference));
        verify(mockProxy).getString(stringVariableValueReference);
    }

    private void configureTest(ScalarVariable variable) {
        when(mockProxy.getReal(anyInt())).thenReturn(new double[]{anyDouble()});
        when(mockedModelDescription.getVariable(variable.getName())).thenReturn(variable);
        mockProxy.initialize(0, 0);
        reader.read(variable.getName());
    }

}
