/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.v2;

import org.javafmi.VariableWriter;
import org.javafmi.modeldescription.ModelDescription;
import org.javafmi.modeldescription.v2.*;
import org.junit.Before;
import org.junit.Test;

import static org.mockito.Mockito.*;

public class VariableWriterTest {

    private org.javafmi.kernel.FmuProxy mockProxy;
    private VariableWriter writer;
    private ModelDescription mockedModelDescription;

    @Before
    public void setUp() {
        mockProxy = mock(org.javafmi.kernel.v2.FmuProxy.class);
        mockedModelDescription = mock(ModelDescription.class);
        writer = new VariableWriter(mockProxy, mockedModelDescription);
    }

    @Test
    public void writeARealVariableCallsToSetRealInProxy() {
        String realVariableName = "";
        double[] realVariableValue = {0.0};
        int[] realVariableValueReference = {0};
        configureTest(new RealVariable(realVariableName).withValueReference(realVariableValueReference[0]), realVariableValue[0]);
        verify(mockProxy).setReal(realVariableValueReference, realVariableValue);
    }

    @Test
    public void writeAIntegerVariableCallsToSetIntegerInProxy() {
        String integerVariableName = "";
        Integer integerVariableValue = 0;
        Integer integerVariableValueReference = 0;
        configureTest(new IntegerVariable(integerVariableName).withValueReference(integerVariableValueReference), integerVariableValue);
        verify(mockProxy).setInteger(integerVariableValueReference, integerVariableValue);
    }

    @Test
    public void writeABooleanVariableCallsToSetBooleanInProxy() {
        String booleanVariableName = "";
        Boolean booleanVariableValue = true;
        Integer booleanVariableValueReference = 0;
        configureTest(new BooleanVariable(booleanVariableName).withValueReference(booleanVariableValueReference), booleanVariableValue);
        verify(mockProxy).setBoolean(booleanVariableValueReference, booleanVariableValue);
    }

    @Test
    public void writeAStringVariableCallsToSetStringInProxy() {
        String stringVariableName = "";
        String stringVariableValue = "";
        Integer stringVariableValueReference = 0;
        configureTest(new StringVariable(stringVariableName).withValueReference(stringVariableValueReference), stringVariableValue);
        verify(mockProxy).setString(stringVariableValueReference, stringVariableValue);
    }

    private void configureTest(ScalarVariable variable, Object value) {
        when(mockedModelDescription.getVariable(variable.getName())).thenReturn(variable);
        writer.write(variable.getName(), value);
    }
}
