/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.modeldescription;

import org.javafmi.modeldescription.v2.Capabilities;
import org.junit.Test;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;

import static org.junit.Assert.*;

public class ModelDescriptionTest {

    private static final String V1_MODEL_DESCRIPTION_PATH = "../test resources/Model Description/v1/";
    private static final String V2_MODEL_DESCRIPTION_PATH = "../test resources/Model Description/v2/";
    private static final String MODEL_DESCRIPTION_FILENAME = "modelDescription.xml";

    @Test
    public void testCheckAllV1ModelDescriptionExamples() throws Exception {
        parseModelDescriptionFiles(getModelDescriptionFiles(new File(V1_MODEL_DESCRIPTION_PATH)), org.javafmi.modeldescription.v1.ModelDescription.class);
    }

    @Test
    public void testCheckAllV2ModelDescriptionExamples() throws Exception {
        parseModelDescriptionFiles(getModelDescriptionFiles(new File(V2_MODEL_DESCRIPTION_PATH)), org.javafmi.modeldescription.v2.ModelDescription.class);
    }

    private ArrayList<File> getModelDescriptionFiles(File file) throws FileNotFoundException {
        if (!file.exists()) throw new FileNotFoundException(file.getPath());
        ArrayList<File> fileList = new ArrayList<>();
        if(file.isDirectory())
            for(File child : file.listFiles())
                fileList.addAll(getModelDescriptionFiles(child));
        else if(file.getName().equals(MODEL_DESCRIPTION_FILENAME))
            fileList.add(file);
        return fileList;
    }

    public void parseModelDescriptionFiles(ArrayList<File> modelDescriptionFiles, Class<? extends ModelDescription> modelDescriptionClass) {
        for(File file : modelDescriptionFiles) {
            try {
                if(ModelDescriptionDeserializer.deserialize(modelDescriptionClass, file) == null) {
                    System.out.println(file.getAbsolutePath());
                    fail();
                }
            } catch(Exception e) {
                System.err.println(file.getAbsolutePath());
                e.printStackTrace();
                fail();
            }
        }
    }

    @Test
    public void whenVariableIsNotDefinedV1ModelDescriptionThrowsAnException() {
        try {
            org.javafmi.modeldescription.v1.ModelDescription modelDescription = new org.javafmi.modeldescription.v1.ModelDescription();
            modelDescription.buildAttributes();
            modelDescription.getVariable("NotDefinedVariable");
            assertTrue("Should throw an exception", false);
        } catch(NoSuchVariableException e) {
            assertTrue(true);
        }
    }

    @Test
    public void whenVariableIsNotDefinedV2ModelDescriptionThrowsAnException() {
        try {
            org.javafmi.modeldescription.v2.ModelDescription modelDescription = new org.javafmi.modeldescription.v2.ModelDescription();
            modelDescription.buildAttributes();
            modelDescription.getVariable("NotDefinedVariable");
            assertTrue("Should throw an exception", false);
        } catch(NoSuchVariableException e) {
            assertTrue(true);
        }
    }

    @Test
    public void gettingV2Capabilities() {
        File modelDescriptionFile = new File(V2_MODEL_DESCRIPTION_PATH + "cosimulation/linux_64_V2_tankv3/" + MODEL_DESCRIPTION_FILENAME);
        org.javafmi.modeldescription.v2.ModelDescription modelDescription = (org.javafmi.modeldescription.v2.ModelDescription) ModelDescriptionDeserializer.deserialize(org.javafmi.modeldescription.v2.ModelDescription.class, modelDescriptionFile);
        Capabilities capabilities = modelDescription.getCapabilities();
        assertEquals(false, capabilities.check(Capabilities.NEEDS_EXECUTION_TOOL));
        assertEquals(true, capabilities.check(Capabilities.CAN_HANDLE_VARIABLE_COMMUNICATION_STEP_SIZE));
        assertEquals(true, capabilities.check(Capabilities.CAN_INTERPOLATE_INPUTS));
        assertEquals(false, capabilities.check(Capabilities.CAN_RUN_ASYNCHRONOUSLY));
        assertEquals(true, capabilities.check(Capabilities.CAN_BE_INSTANTIATED_ONLY_ONCE_PER_PROCESS));
        assertEquals(false, capabilities.check(Capabilities.CAN_NOT_USE_MEMORY_MANAGEMENT_FUNCTIONS));
        assertEquals(false, capabilities.check(Capabilities.CAN_SERIALIZE_FMU_STATE));
        assertEquals(true, capabilities.check(Capabilities.PROVIDES_DIRECTIONAL_DERIVATIVE));
    }
}
