/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.exporter.framework.fmiInterface;

import org.javafmi.kernel.Status;
import org.javafmi.modeldescription.v2.ModelDescription;


public abstract class FmiSimulation extends FmiAccess {
    public FmiSimulation(ModelDescription modelDescription) {
        super(modelDescription);
    }

    @Override
    public Status enterInitializationMode() {
        return Status.OK;
    }

    @Override
    public Status exitInitializationMode() {
        return init();
    }

    public Status setupExperiment(boolean toleranceDefined, double tolerance, double startTime, boolean stopTimeDefined, double stopTime) {
        return Status.OK;
    }

    @Override
    public double[] getReal(String variableName[]) {
        double doubleValues[] = new double[variableName.length];
        for(int i = 0; i < variableName.length; i++) {
            doubleValues[i] = (double) getVariable(variableName[i]);
        }
        return doubleValues;
    }

    @Override
    public int[] getInteger(String variableName[]) {
        int intValues[] = new int[variableName.length];
        for(int i = 0; i < variableName.length; i++) {
            intValues[i] = (int) getVariable(variableName[i]);
        }
        return intValues;
    }

    @Override
    public boolean[] getBoolean(String variableName[]) {
        boolean boolValues[] = new boolean[variableName.length];
        for(int i = 0; i < variableName.length; i++) {
            boolValues[i] = (boolean) getVariable(variableName[i]);
        }
        return boolValues;
    }

    @Override
    public String[] getString(String variableName[]) {
        String stringValues[] = new String[variableName.length];
        for(int i = 0; i < variableName.length; i++) {
            stringValues[i] = (String) getVariable(variableName[i]);
        }
        return stringValues;
    }

    @Override
    public Status setReal(String valueReferenceName[], double values[]) {
        for(int i = 0; i < valueReferenceName.length; i++) {
            setVariable(valueReferenceName[i], values[i]);
        }
        return Status.OK;
    }

    @Override
    public Status setInteger(String valueReferenceName[], int values[]) {
        for(int i = 0; i < valueReferenceName.length; i++) {
            setVariable(valueReferenceName[i], values[i]);
        }
        return Status.OK;
    }

    @Override
    public Status setBoolean(String valueReferenceName[], boolean values[]) {
        for(int i = 0; i < valueReferenceName.length; i++) {
            setVariable(valueReferenceName[i], values[i]);
        }
        return Status.OK;
    }

    @Override
    public Status setString(String valueReferenceName[], String values[]) {
        for(int i = 0; i < valueReferenceName.length; i++) {
            setVariable(valueReferenceName[i], values[i]);
        }
        return Status.OK;
    }

    @Override
    public Status doStep(double currentCommunicationPoint, double communicationStepSize, boolean newStep) {
        return doStep(communicationStepSize);
    }

    @Override
    public Status cancelStep() {
        return Status.WARNING;
    }

    @Override
    public int fmiReset() {
        return reset().getCode();
    }

    @Override
    public double[] getDirectionalDerivative(String[] nameUnknowns, String[] nameKnowns, double[] dvKnown) {
        return new double[]{-1.0};
    }

    @Override
    public double[] getRealOutputDerivatives(String[] nameVariable, int[] order) {
        return new double[]{-1.0};
    }

    @Override
    public Status setRealInputDerivatives(String[] nameVariable, int[] order, double[] values) {
        return Status.OK;
    }

    public abstract Status init();

    public abstract Object getVariable(String nameVariable);

    public abstract Status setVariable(String nameVariable, Object values);

    public abstract Status doStep(double stepSize);

    public abstract Status reset();

    public abstract Status terminate();

}
