/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.exporter.fmubuilder;

import org.javafmi.kernel.operatingsystem.PathBuilder;
import org.javafmi.kernel.operatingsystem.UnZipper;

import java.io.File;
import java.io.IOException;
import java.util.jar.JarEntry;
import java.util.jar.JarOutputStream;

import static org.javafmi.kernel.operatingsystem.FileUtils.*;

public class JarBuilder {

    private File workingDir;
    private String jarName;

    public JarBuilder(String jarName) {
        this.jarName = (jarName.endsWith(".jar") ? jarName : jarName + ".jar");
        this.workingDir = createTemporal();
    }

    public JarBuilder addClassesFromJar(File... jarSources) {
        for(File jarSource : jarSources) {
            new UnZipper(jarSource).unzipAll(workingDir);
        }
        return this;
    }

    public File buildJar() {
        JarOutputStream jarOutputStream = jarOutputStream(fileOutputStream(new File(jarName)));
        addToJar(workingDir, jarOutputStream);
        close(jarOutputStream);
        deleteAll(workingDir);
        return new File(jarName);
    }

    private File createTemporal() {
        String systemTempDirectory = System.getProperty("java.io.tmpdir");
        File tempFolder = new File(PathBuilder.buildPath(systemTempDirectory, "rt_fmu" + System.currentTimeMillis()));
        tempFolder.mkdirs();
        tempFolder.deleteOnExit();
        return tempFolder;
    }

    private void addToJar(File nextFile, JarOutputStream jarOutputStream) {
        if(nextFile.isDirectory()) addNextFileAsDirectory(nextFile, jarOutputStream);
        else addNextFileAsEntry(nextFile, jarOutputStream);
    }

    private void addNextFileAsDirectory(File nextFile, JarOutputStream jarOutputStream) {

         /*   String entryName = replaceBackSlashes(nextFile.getPath());
            if(!entryName.endsWith("/")) entryName += "/";
            jarOutputStream.putNextEntry(new JarEntry("rt/"));
            jarOutputStream.closeEntry();*/
        for(File childFile : nextFile.listFiles()) {
            addToJar(childFile, jarOutputStream);
        }

    }

    private void addNextFileAsEntry(File nextFile, JarOutputStream jarOutputStream) {
        try {
            String entryName = replaceBackSlashes(nextFile.getPath());
            jarOutputStream.putNextEntry(new JarEntry(entryName.substring((entryName.indexOf(workingDir.getName()) + workingDir.getName().length() + 1), entryName.length())));
            copyFromTo(bufferedInputStream(fileInputStream(nextFile)), jarOutputStream);
            jarOutputStream.closeEntry();
        } catch(IOException e) {
            e.printStackTrace();
        }
    }

    private String replaceBackSlashes(String path) {
        String newName = path.replaceAll("\\\\", "/");
        return newName.substring(newName.indexOf('/') + 1);
    }
}
