/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI org.javafmi.fmubuilder.Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.exporter.fmubuilder;

import org.javafmi.kernel.operatingsystem.UnZipper;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.nio.file.Files;
import java.nio.file.Path;

import static org.javafmi.kernel.operatingsystem.FileUtils.close;
import static org.javafmi.kernel.operatingsystem.FileUtils.copyFromTo;

public class JarClassLoader extends ClassLoader {
    private File jarFile;
    private File outFolder;

    public JarClassLoader(File jarFile) {
        this.jarFile = jarFile;
        this.outFolder = getTempDirectory().toFile();
    }

    public Path getTempDirectory() {
        try {
            return Files.createTempDirectory(getClass().getName());
        } catch(IOException e) {
            throw new RuntimeException("Impossible to create temp directory to extract " + jarFile.getName());
        }
    }

    public Class loadClass(String className) {
        if(isAJavaFMIClass(className) || isAJavaClass(className)) return loadCLassUsingSuperClassLoader(className);
        byte[] classData = getExtractClassBytes(getClassUrl(replaceDotWithSlash(className)));
        return defineClass(className, classData, 0, classData.length);
    }

    public boolean isAJavaFMIClass(String className) {
        return className.startsWith("org.javafmi");
    }

    public boolean isAJavaClass(String className) {
        return className.startsWith("java");
    }

    private Class loadCLassUsingSuperClassLoader(String className) {
        try {
            return super.loadClass(className);
        } catch(ClassNotFoundException e) {
            throw new RuntimeException(e);
        }
    }

    private String replaceDotWithSlash(String className) {
        while(className.indexOf('.') > 0)
            className = className.replace('.', '/');
        return className;
    }

    public URL getClassUrl(String className) {
        try {
            return new UnZipper(jarFile).unzip(className + ".class", outFolder).toURI().toURL();
        } catch(IOException e) {
            throw new RuntimeException("Problem getting the class URL " + className);
        }
    }

    private byte[] getExtractClassBytes(URL classUrl) {
        InputStream input = getInputStreamFromConnection(classUrl);
        ByteArrayOutputStream buffer = new ByteArrayOutputStream();
        copyFromTo(input, buffer);
        close(input);
        return buffer.toByteArray();
    }

    private URLConnection openConnection(URL classUrl) {
        try {
            return classUrl.openConnection();
        } catch(IOException e) {
            throw new RuntimeException(e);
        }
    }

    private InputStream getInputStreamFromConnection(URL classUrl) {
        try {
            return openConnection(classUrl).getInputStream();
        } catch(IOException e) {
            throw new RuntimeException(e);
        }
    }
}
