/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.exporter.fmubuilder;

import org.junit.After;
import org.junit.Test;

import java.io.File;

import static org.javafmi.kernel.operatingsystem.FileUtils.deleteAll;
import static org.junit.Assert.*;

public class JarBuilderTest {

    public static final String JAR_EXTENSION = ".jar";
    public static final String SIMPLE_XML_2_7_1_JAR = "../libraries/simple-xml-2.7.1.jar";
    public static final String JNA_3_5_2_JAR = "../libraries/jna-3.5.2.jar";
    public static final String JAVA_FMI_V2_5_BETA_JAR = "../libraries/JavaFMI/JavaFMI v2.5 beta for FMI V2RC2.jar";
    private static final long EMPTY_JAR_LENGTH = 136;
    private String jarName;

    @After
    public void setUp() {
        deleteAll(new File(jarName + JAR_EXTENSION));
    }

    @Test
    public void creationOfEmptyJarWithTheProperName() {
        jarName = "empty";
        File createdJar = new JarBuilder(jarName).buildJar();
        assertTrue(createdJar.exists());
        assertEquals(EMPTY_JAR_LENGTH, createdJar.length());
    }

    @Test
    public void runsAnApplicationWhichUsesTheClassesInTheCreatedJar() {
        try {
            jarName = "rt";
            JarBuilder jarBuilder = new JarBuilder(jarName);
            jarBuilder.addClassesFromJar(
                    new File(SIMPLE_XML_2_7_1_JAR),
                    new File(JNA_3_5_2_JAR),
                    new File(JAVA_FMI_V2_5_BETA_JAR),
                    new File("test resources/BouncingBall_forJarBuilderTest.jar"));
            File rtJar = jarBuilder.buildJar();
            Process start = new ProcessBuilder("java", "-cp", rtJar.getAbsolutePath(), "Main").start();
            start.waitFor();
            assertTrue(start.exitValue() == 0);
        } catch(Exception e) {
            e.printStackTrace();
            fail();
        }
    }

}
