/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI org.javafmi.fmubuilder.Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.exporter.fmubuilder;


import org.javafmi.modeldescription.v2.ModelDescription;
import org.junit.Before;
import org.junit.Test;

import java.io.File;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

public class JarClassLoaderTest {
    public static final String JAR_PATH = "test resources/ForJarClassLoaderTest.jar";
    public static final String SIMPLE_CLASS = "SimpleClass";
    public static final String SUB_CLASS = "SubClass";
    public static final String VARIABLE_NAME = "Variable";
    private final static String MODEL_DESCRIPTION_CLASS = "MyModelDescription";
    private JarClassLoader loader;

    @Before
    public void setUp() {
        loader = new JarClassLoader(new File(JAR_PATH));
    }

    @Test
    public void gettingASimpleClassFromJar() throws IllegalAccessException, InstantiationException {
        Object object = loader.loadClass(SIMPLE_CLASS).newInstance();
        assertEquals("SimpleClassForTest", object.toString());
    }

    @Test
    public void gettingASubClass() throws IllegalAccessException, InstantiationException {
        Object object = loader.loadClass(SUB_CLASS).newInstance();
        assertEquals("SubClassForTest", object.toString());
    }

    @Test
    public void gettingASubClassWhichCallsToParentClass() throws IllegalAccessException, InstantiationException {
        ModelDescription modelDescription = (ModelDescription) loader.loadClass(MODEL_DESCRIPTION_CLASS).newInstance();
        assertNotNull(modelDescription.getVariable(VARIABLE_NAME));
    }
}
