#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "PathUtilities.h"

#if (defined __DARWIN64)
	#define SYSTEM_NAME "darwin"
	#define SYSTEM_ARCH "64"
	#define LIBRARY_EXTENSION "dylib"
#elif (defined __WIN64)
	#define SYSTEM_NAME "win"
	#define SYSTEM_ARCH "64"
	#define LIBRARY_EXTENSION "dll"
#elif (defined __WIN32)
	#define SYSTEM_NAME "win"
	#define SYSTEM_ARCH "32"
	#define LIBRARY_EXTENSION "dll"
#elif (defined __LINUX64)
	#define SYSTEM_NAME "linux"
	#define SYSTEM_ARCH "64"
	#define LIBRARY_EXTENSION "so"
#endif


char* removeSchemeName(char* url){
	int slashCounter = 0;
	char* result = url;
	while(slashCounter < 2){
		if(*(result++) == '/'){ slashCounter++; };
	}
	return result;
}

int extractResourcesPath(const char* resourceURL, char* output){
	char* resourcesPath = removeSchemeName((char*) resourceURL);
	memcpy(output, resourcesPath, strlen(resourcesPath));
	return (output[strlen(output) - 1] == '/');
}

int getBootstrap(const char* resourcesPath, char* output){
	char* parentDirectoryPath = (char*) calloc(256, sizeof(char));
	getParentDirectory((char*) resourcesPath, parentDirectoryPath);
 	int result = sprintf(output,"-Davian.bootstrap=%sbinaries/%s/lib%sShell.%s", parentDirectoryPath, SYSTEM_NAME SYSTEM_ARCH, SYSTEM_NAME SYSTEM_ARCH, LIBRARY_EXTENSION) > 0;
	free(parentDirectoryPath);
	return result;

}

int getBootClasspath(const char* resourcesPath, char* output){
	return sprintf(output, "-Xbootclasspath:%s/rt.jar", resourcesPath) > 0;
}

int getJavaClass(const char* resourcesPath, char* output){
    return sprintf(output, "-Djava.class.path=%s/rt.jar", resourcesPath) > 0;
}

int getParentDirectory(char* directory, char* output){
	char* slashPosition = &directory[strlen(directory) - 2];
	for (int i = strlen(directory) - 2; i > 0; i--){
		if(directory[i] != '/') slashPosition--;
		else break;
	}
	return memcpy(output, directory, strlen(directory) - strlen(slashPosition + 1)) > 0;
}
