#include "fmi/fmiFunctions.h"
#include "fmi/fmiStatusUtilities.h"
#include "jvm/VirtualMachine.h"
#include "PathUtilities.h"

const char* fmiGetTypesPlatform(){
	return "default";
}

const char* fmiGetVersion(){
	return "2.0";
}

fmiStatus fmiSetDebugLogging(fmiComponent component, fmiBoolean loggingOn, size_t numberOfCategories, const fmiString categories[]){
	return fmiError;
}

fmiComponent fmiInstantiate(fmiString instanceName, fmiType fmuType, fmiString fmuGUID, fmiString fmuResourceLocation, const fmiCallbackFunctions* callbacksFunctions, fmiBoolean visible, fmiBoolean loggingOn){
	char* resourcesPath = (char*) calloc(256, sizeof(char));
	char* bootstrap = (char*) calloc(256, sizeof(char));
	char* bootClasspath = (char*) calloc(256, sizeof(char));
	extractResourcesPath(fmuResourceLocation, resourcesPath);
	getBootstrap(resourcesPath, bootstrap);
	getBootClasspath(resourcesPath, bootClasspath);
	void* virtualMachine = createVirtualMachine(bootstrap, bootClasspath);
	loadMainObject(virtualMachine, instanceName);
	free(resourcesPath);
	free(bootstrap);
	free(bootClasspath);
	return virtualMachine;
}

fmiStatus fmiSetupExperiment(fmiComponent component, fmiBoolean toleranceDefined, fmiReal tolerance, fmiReal startTime, fmiBoolean stopTimeDefined, fmiReal stopTime){
	return getFmiStatusCode(invokeSetupExperiment(component, toleranceDefined, tolerance, startTime, stopTimeDefined, stopTime));
}

fmiStatus fmiEnterInitializationMode(fmiComponent component){
	return getFmiStatusCode(invokeEnterInitializationMode(component));
}

fmiStatus fmiExitInitializationMode(fmiComponent component ){
	return getFmiStatusCode(invokeExitInitializationMode(component));
}

fmiStatus fmiTerminate(fmiComponent component){
	return getFmiStatusCode(invokeTerminate(component));
}

fmiStatus fmiFreeInstance(fmiComponent component){
	return fmiOK;
}

fmiStatus fmiReset(fmiComponent component){
	return getFmiStatusCode(resetVirtualMachine(component));
}

fmiStatus fmiGetReal(fmiComponent component, const fmiValueReference valueReference[], size_t numberOfValues, fmiReal values[]){
	invokeGetReal(component, (int*)valueReference, numberOfValues, values);
	return fmiOK;
}

fmiStatus fmiGetInteger(fmiComponent component, const fmiValueReference valueReference[], size_t numberOfValues, fmiInteger values[]){
	invokeGetInteger(component, (int*)valueReference, numberOfValues, values);
	return fmiOK;
}

fmiStatus fmiGetBoolean(fmiComponent component, const fmiValueReference valueReference[], size_t numberOfValues, fmiBoolean values[]){
	invokeGetBoolean(component, (int*)valueReference, numberOfValues, values);
	return fmiOK;
}

fmiStatus fmiGetString(fmiComponent component, const fmiValueReference valueReference[], size_t numberOfValues, fmiString values[]){
	invokeGetString(component, (int*)valueReference, numberOfValues, (char**)values);
	return fmiOK;
}

fmiStatus fmiSetReal(fmiComponent component, const fmiValueReference valueReference[], size_t numberOfValues, const fmiReal values[]){
	 return getFmiStatusCode(invokeSetReal(component, (int*)valueReference, numberOfValues, (double*)values));	
}

fmiStatus fmiSetInteger(fmiComponent component, const fmiValueReference valueReference[], size_t numberOfValues, const fmiInteger values[]){
	return getFmiStatusCode(invokeSetInteger(component, (int*)valueReference, numberOfValues, (int*)values));
}

fmiStatus fmiSetBoolean(fmiComponent component, const fmiValueReference valueReference[], size_t numberOfValues, const fmiBoolean values[]){
	return getFmiStatusCode(invokeSetBoolean(component, (int*)valueReference, numberOfValues, (int*)values));
}

fmiStatus fmiSetString(fmiComponent component, const fmiValueReference valueReference[], size_t numberOfValues, const fmiString values[]){
	return getFmiStatusCode(invokeSetString(component, (int*)valueReference, numberOfValues, (char**) values));
}

fmiStatus fmiGetFMUstate(fmiComponent component, fmiFMUstate* state){
	return fmiWarning;
}

fmiStatus fmiSetFMUstate(fmiComponent component, fmiFMUstate state){
	return fmiWarning;
}

fmiStatus fmiFreeFMUstate(fmiComponent component, fmiFMUstate* state){
	return fmiWarning;
}

fmiStatus fmiSerializedFMUstateSize(fmiComponent component, fmiFMUstate, size_t* stateSize){
	return fmiWarning;
}

fmiStatus fmiSerializeFMUstate(fmiComponent component, fmiFMUstate state, fmiByte serializedState[], size_t serializedStateSize){
	return fmiWarning;
}

fmiStatus fmiDeSerializeFMUstate(fmiComponent component, const fmiByte serializedState[], size_t serializedStateSize, fmiFMUstate* state){
	return fmiWarning;
}

fmiStatus fmiGetDirectionalDerivative(fmiComponent component, const fmiValueReference unknownReference[], size_t numberOfUnknowns, const fmiValueReference knownReferences[], size_t numberOfKnowns, const fmiReal dvKnown[], fmiReal dvUnKnown[]){
	invokeGetDirectionalDerivative(component, (int*) unknownReference, numberOfUnknowns, (int*) knownReferences, numberOfKnowns, (double*) dvKnown, (double*) dvUnKnown);
	return fmiOK;
}

fmiStatus fmiSetRealInputDerivatives(fmiComponent component, const fmiValueReference valueReference[], size_t numberOfValues, const fmiInteger order[], const fmiReal values[]){
	return getFmiStatusCode(invokeSetRealInputDerivatives(component, (int*) valueReference, numberOfValues, (int*) order, (double*) values));
}

fmiStatus fmiGetRealOutputDerivatives(fmiComponent component, const fmiValueReference valueReference[], size_t numberOfValues, const fmiInteger order[], fmiReal values[]){
	invokeGetRealOutputDerivatives(component, (int*) valueReference, numberOfValues, (int*) order, (double*) values);
	return fmiOK;
}

fmiStatus fmiDoStep(fmiComponent component, fmiReal currentComunicationTime, fmiReal stepSize, fmiBoolean noSetFmuFmuStatePriorToCurrentPoint){
	return getFmiStatusCode(invokeDoStep(component, (double)currentComunicationTime, (double)stepSize, (int)noSetFmuFmuStatePriorToCurrentPoint));
}

fmiStatus fmiCancelStep(fmiComponent component){
	return getFmiStatusCode(invokeCancelStep(component));
}

fmiStatus fmiGetStatus(fmiComponent component, const fmiStatusKind kind, fmiStatus* status){
	return fmiWarning;	
}

fmiStatus fmiGetRealStatus(fmiComponent component, const fmiStatusKind kind, fmiReal* value){
	return fmiWarning;
}

fmiStatus fmiGetIntegerStatus(fmiComponent component, const fmiStatusKind kind, fmiInteger* value){
	return fmiWarning;
}

fmiStatus fmiGetBooleanStatus(fmiComponent component, const fmiStatusKind kind, fmiBoolean* value){
	return fmiWarning;
}

fmiStatus fmiGetStringStatus(fmiComponent component, const fmiStatusKind kind, fmiString* value){
	return fmiWarning;
}

