#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "../src/PathUtilities.h"
#include "../lib/cutest/CuTest.h"

const char* getExpectedValueForThisSO(){
#if (defined __DARWIN64)
	const char* expectedValue = "-Davian.bootstrap=/this/is/a/path/to/fmu/binaries/darwin64/libdarwin64Shell.dylib";
#elif (defined __WIN32)
	const char* expectedValue = "-Davian.bootstrap=/this/is/a/path/to/fmu/binaries/win32/libwin32Shell.dll";
#endif
	return expectedValue;
}

void gets_bootstrap_with_the_path_to_the_library(CuTest* cutest){
	const char* input = "/this/is/a/path/to/fmu/resources/";
	char* result = (char*) calloc(256, sizeof(char));
	const char* expectedValue = getExpectedValueForThisSO();
	CuAssertIntEquals(cutest, 1, getBootstrap(input, result));	
	CuAssertStrEquals(cutest, expectedValue, result);
	free(result);
}

void retrieve_resources_path_from_url(CuTest* cutest){
	const char* input = "file:///this/is/a/path/to/resources/";
	char* result = (char*) calloc(strlen(input) + 1, sizeof(char));
	const char* expectedValue = "/this/is/a/path/to/resources/";
	CuAssertIntEquals(cutest, 1, extractResourcesPath(input, result));
	CuAssertStrEquals(cutest, expectedValue, result);
	free(result);
}

void returns_error_when_malformed_url(CuTest* cutest){
	const char* input = "file:///bad/formed/path/to/resources";
	char* result = (char*) calloc(strlen(input) + 1, sizeof(char));
	const char* expectedValue = "/this/is/a/path/to/resources/";
	CuAssertIntEquals(cutest, 0, extractResourcesPath(input, result));
	free(result);
}

void get_parent_directory_should_return_parent_directory(CuTest* cutest){
	const char* input = "/this/is/a/path/to/fmu/resources/";
	char* result = (char*) calloc(strlen(input) + 1, sizeof(char));
	const char* expectedValue = "/this/is/a/path/to/fmu/";
	CuAssertIntEquals(cutest, 1, getParentDirectory((char*)input, result));	
	CuAssertStrEquals(cutest, expectedValue, result);
	free(result);
}

void gets_the_bootclasspath_with_the_runtime_jar_path(CuTest* cutest){
	const char* input = "/this/is/a/path/to/fmu/resources/";
	char* result = (char*) calloc(256, sizeof(char));
	const char* expectedValue = "-Xbootclasspath:/this/is/a/path/to/fmu/resources/rt.jar";
	CuAssertIntEquals(cutest, 1, getBootClasspath(input,result));
	CuAssertStrEquals(cutest, expectedValue, result);
	free(result);
}

CuSuite* loadPathUtilitiesSuite(){	
	CuSuite* suite = CuSuiteNew();
	SUITE_ADD_TEST(suite, retrieve_resources_path_from_url);
	SUITE_ADD_TEST(suite, returns_error_when_malformed_url);
	SUITE_ADD_TEST(suite, gets_bootstrap_with_the_path_to_the_library);
	SUITE_ADD_TEST(suite, gets_the_bootclasspath_with_the_runtime_jar_path);
	return suite;
}
