/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */

import org.javafmi.exporter.framework.fmiInterface.FmiSimulation;
import org.javafmi.exporter.framework.fmiInterface.Variable;
import org.javafmi.exporter.framework.fmiInterface.VariableHandler;
import org.javafmi.kernel.Status;

public class Trip extends FmiSimulation {

    private Car carForTrip;
    private VariableHandler variableHandler;

    public Trip() {
        this(new Car(new Odometer(0.)));
    }

    public Trip(Car carForTrip) {
        super(new FmiModelDescription("Trip"));
        this.carForTrip = carForTrip;
        this.variableHandler = new VariableHandler();
        this.variableHandler.register(FmiModelDescription.CAR_SPEED, new Variable(carForTrip, "speed"));
        this.variableHandler.register(FmiModelDescription.ODOMETER_DISTANCE, new Variable(carForTrip.getOdometer(), "distance"));
    }

    @Override
    public Status setVariable(String variableName, Object value) {
        variableHandler.set(variableName, value);
        return Status.OK;
    }

    @Override
    public Status init() {
        return Status.OK;
    }

    @Override
    public Object getVariable(String variableName) {
        return variableHandler.get(variableName).getValue();
    }

    @Override
    public Status doStep(double stepSize) {
        carForTrip.advance(stepSize);
        return Status.OK;
    }

    @Override
    public Status terminate() {
        return Status.OK;
    }

    @Override
    public Status reset() {
        return Status.OK;
    }
}
