/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */

import org.javafmi.exporter.framework.fmiInterface.FmiSimulation;
import org.javafmi.kernel.Status;
import org.junit.Test;

import static org.junit.Assert.assertEquals;

public class FmiSimulationTest {

    @Test
    public void createsTheTripSimulation() {
        Odometer odometer = new Odometer(0.);
        Car carForTrip = new Car(odometer);
        FmiSimulation trip = new Trip(carForTrip);
        trip.init();

        trip.setVariable(FmiModelDescription.CAR_SPEED, 40);
        trip.doStep(1800);
        Double distanceAfterHalfHour = (Double) trip.getVariable(FmiModelDescription.ODOMETER_DISTANCE);

        assertEquals(odometer.getDistance(), distanceAfterHalfHour);
        assertEquals(20000, distanceAfterHalfHour, 0.1);

        trip.doStep(1800);
        Double distanceAfterAnHour = (Double) trip.getVariable(FmiModelDescription.ODOMETER_DISTANCE);
        assertEquals(40000, distanceAfterAnHour, 0.1);
        Status terminateStatus = trip.terminate();
        assertEquals(Status.OK, terminateStatus);
    }

}
